import { ComponentProps, JSX, Ref } from 'solid-js';
import { Grammar, Theme } from './tm';

export { css } from './css';
/**********************************************************************************/
/**********************************************************************************/
export interface TmTextareaProps extends Omit<ComponentProps<'div'>, 'style' | 'onInput' | 'onScroll'> {
    /** If textarea is editable or not. */
    editable?: boolean;
    /** The grammar of the source code for syntax highlighting. */
    grammar: Grammar;
    /** Custom CSS properties to apply to the editor. */
    style?: JSX.CSSProperties;
    /** Ref to the internal html-textarea-element. */
    textareaRef?: Ref<HTMLTextAreaElement>;
    /** The theme to apply for syntax highlighting. */
    theme: Theme;
    /** The source code to be displayed and edited. */
    value: string;
    /** Callback function to handle input-event. */
    onInput?: (event: InputEvent & {
        currentTarget: HTMLTextAreaElement;
    }) => void;
    /** Callback function to handle scroll-event. */
    onScroll?: (event: Event & {
        currentTarget: HTMLDivElement;
    }) => void;
}
export declare function createTmTextarea(styles: Record<string, string>): (props: TmTextareaProps) => JSX.Element;
